from .debug import *  # NOQA

ALLOWED_HOSTS = ['testserver']

# test raven integration:

# INSTALLED_APPS += ['raven.contrib.django']
#
# RAVEN_CONFIG = {
#     'dsn': os.environ.get('SENTRY_DSN'),
# }
