# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-29 16:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('yawn', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queue',
            name='name',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='task',
            name='run',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='yawn.Run'),
        ),
        migrations.AlterField(
            model_name='template',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='template',
            name='workflow',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT,
                                    to='yawn.Workflow'),
        ),
        migrations.AlterField(
            model_name='workflowname',
            name='name',
            field=models.TextField(unique=True),
        ),
    ]
